@@ -9,7 +9,8 @@
 )))]
 use super::conv::ret_u32;
 use super::conv::{
-    borrowed_fd, no_fd, ret, ret_c_int, ret_discarded_fd, ret_off_t, ret_owned_fd, ret_ssize_t,
+    borrowed_fd, c_str, no_fd, ret, ret_c_int, ret_discarded_char_ptr, ret_discarded_fd, ret_off_t,
+    ret_owned_fd, ret_ssize_t,
 };
 #[cfg(any(target_os = "android", target_os = "linux"))]
 use super::conv::{syscall_ret, syscall_ret_owned_fd, syscall_ret_ssize_t};
@@ -51,8 +52,9 @@ use super::io::PollFd;
 use super::io::ReadWriteFlags;
 #[cfg(not(any(target_os = "redox", target_os = "wasi")))]
 use super::net::{
-    decode_sockaddr, AcceptFlags, AddressFamily, Protocol, RecvFlags, SendFlags, Shutdown,
-    SocketAddr, SocketAddrUnix, SocketAddrV4, SocketAddrV6, SocketType,
+    encode_sockaddr_unix, encode_sockaddr_v4, encode_sockaddr_v6, read_sockaddr_os, AcceptFlags,
+    AddressFamily, Protocol, RecvFlags, SendFlags, Shutdown, SocketAddr, SocketAddrUnix,
+    SocketFlags, SocketType,
 };
 #[cfg(any(target_os = "android", target_os = "fuchsia", target_os = "linux"))]
 use super::offset::libc_fallocate;
@@ -97,33 +99,29 @@ use libc::{c_int, c_void};
 use std::cmp::min;
 use std::convert::TryInto;
 use std::ffi::CStr;
-#[cfg(not(any(target_os = "fuchsia", target_os = "wasi")))]
-use std::ffi::OsString;
 use std::io::{IoSlice, IoSliceMut, SeekFrom};
 #[cfg(target_os = "linux")]
 use std::mem::transmute;
 use std::mem::{size_of, MaybeUninit};
-#[cfg(all(unix, not(target_os = "fuchsia")))]
+use std::net::{SocketAddrV4, SocketAddrV6};
+#[cfg(any(target_os = "ios", target_os = "macos"))]
 use std::os::unix::ffi::OsStringExt;
-#[cfg(target_os = "wasi")]
-use std::os::wasi::ffi::OsStringExt;
 #[cfg(not(any(target_os = "redox", target_os = "wasi",)))]
 use std::ptr::null_mut;
 #[cfg(not(any(target_os = "redox", target_env = "newlib")))]
 use std::sync::atomic::{AtomicUsize, Ordering};
-#[cfg(not(target_os = "redox"))]
-use {
-    super::conv::c_str,
-    super::fs::AtFlags,
-    super::offset::{libc_openat, libc_preadv, libc_pwritev},
-    crate::time::NanosleepRelativeResult,
-};
 #[cfg(any(target_os = "ios", target_os = "macos"))]
 use {
     super::conv::nonnegative_ret,
     super::fs::{copyfile_state_t, CloneFlags, CopyfileFlags},
     std::path::PathBuf,
 };
+#[cfg(not(target_os = "redox"))]
+use {
+    super::fs::AtFlags,
+    super::offset::{libc_openat, libc_preadv, libc_pwritev},
+    crate::time::NanosleepRelativeResult,
+};
 #[cfg(any(target_os = "android", target_os = "linux"))]
 use {
     super::fs::MemfdFlags,
@@ -549,6 +547,16 @@ pub(crate) fn utimensat(
     }
 }
 
+#[cfg(not(target_os = "wasi"))]
+pub(crate) fn chdir(path: &CStr) -> io::Result<()> {
+    unsafe { ret(libc::chdir(c_str(path))) }
+}
+
+#[cfg(not(any(target_os = "wasi", target_os = "fuchsia")))]
+pub(crate) fn fchdir(dirfd: BorrowedFd<'_>) -> io::Result<()> {
+    unsafe { ret(libc::fchdir(borrowed_fd(dirfd))) }
+}
+
 #[cfg(not(any(
     target_os = "android",
     target_os = "linux",
@@ -771,8 +779,8 @@ pub(crate) fn fcntl_get_seals(fd: BorrowedFd<'_>) -> io::Result<u32> {
 }
 
 #[cfg(not(target_os = "wasi"))]
-pub(crate) fn fcntl_dupfd_cloexec(fd: BorrowedFd<'_>) -> io::Result<OwnedFd> {
-    unsafe { ret_owned_fd(libc::fcntl(borrowed_fd(fd), libc::F_DUPFD_CLOEXEC)) }
+pub(crate) fn fcntl_dupfd_cloexec(fd: BorrowedFd<'_>, min: RawFd) -> io::Result<OwnedFd> {
+    unsafe { ret_owned_fd(libc::fcntl(borrowed_fd(fd), libc::F_DUPFD_CLOEXEC, min)) }
 }
 
 pub(crate) fn seek(fd: BorrowedFd<'_>, pos: SeekFrom) -> io::Result<u64> {
@@ -1169,30 +1177,19 @@ pub(crate) fn dup2_with(fd: BorrowedFd<'_>, new: &OwnedFd, _flags: DupFlags) ->
     dup2(fd, new)
 }
 
-#[cfg(not(any(target_os = "fuchsia", target_os = "wasi")))]
-pub(crate) fn ttyname(dirfd: BorrowedFd<'_>, reuse: OsString) -> io::Result<OsString> {
-    // This code would benefit from having a better way to read into
-    // uninitialized memory, but that requires `unsafe`.
-    let mut buffer = reuse.into_vec();
-    buffer.clear();
-    buffer.resize(256, 0_u8);
+#[cfg(not(target_os = "wasi"))]
+pub(crate) fn getcwd(buf: &mut [u8]) -> io::Result<()> {
+    unsafe { ret_discarded_char_ptr(libc::getcwd(buf.as_mut_ptr().cast::<_>(), buf.len())) }
+}
 
-    loop {
-        match unsafe {
-            libc::ttyname_r(
-                borrowed_fd(dirfd),
-                buffer.as_mut_ptr().cast::<libc::c_char>(),
-                buffer.len(),
-            )
-        } {
-            libc::ERANGE => buffer.resize(buffer.len() * 2, 0_u8),
-            0 => {
-                let len = buffer.iter().position(|x| *x == b'\0').unwrap();
-                buffer.resize(len, 0_u8);
-                return Ok(OsString::from_vec(buffer));
-            }
-            errno => return Err(io::Error(errno)),
-        }
+#[cfg(not(any(target_os = "fuchsia", target_os = "wasi")))]
+pub(crate) fn ttyname(dirfd: BorrowedFd<'_>, buf: &mut [u8]) -> io::Result<()> {
+    unsafe {
+        ret(libc::ttyname_r(
+            borrowed_fd(dirfd),
+            buf.as_mut_ptr().cast::<_>(),
+            buf.len(),
+        ))
     }
 }
 
@@ -1440,7 +1437,10 @@ pub(crate) fn recvfrom(
             storage.as_mut_ptr().cast::<_>(),
             &mut len,
         ))?;
-        Ok((nread as usize, decode_sockaddr(storage.as_ptr(), len)))
+        Ok((
+            nread as usize,
+            read_sockaddr_os(storage.as_ptr(), len.try_into().unwrap()),
+        ))
     }
 }
 
@@ -1457,7 +1457,7 @@ pub(crate) fn sendto_v4(
             buf.as_ptr().cast::<_>(),
             buf.len(),
             flags.bits(),
-            as_ptr(&addr.encode()).cast::<libc::sockaddr>(),
+            as_ptr(&encode_sockaddr_v4(addr)).cast::<libc::sockaddr>(),
             size_of::<SocketAddrV4>() as u32,
         ))?
     };
@@ -1477,7 +1477,7 @@ pub(crate) fn sendto_v6(
             buf.as_ptr().cast::<_>(),
             buf.len(),
             flags.bits(),
-            as_ptr(&addr.encode()).cast::<libc::sockaddr>(),
+            as_ptr(&encode_sockaddr_v6(addr)).cast::<libc::sockaddr>(),
             size_of::<SocketAddrV6>() as u32,
         ))?
     };
@@ -1497,7 +1497,7 @@ pub(crate) fn sendto_unix(
             buf.as_ptr().cast::<_>(),
             buf.len(),
             flags.bits(),
-            as_ptr(&addr.encode()).cast::<libc::sockaddr>(),
+            as_ptr(&encode_sockaddr_unix(addr)).cast::<libc::sockaddr>(),
             size_of::<SocketAddrUnix>() as u32,
         ))?
     };
@@ -1514,7 +1514,23 @@ pub(crate) fn socket(
         ret_owned_fd(libc::socket(
             domain.0 as c_int,
             type_.0 as c_int,
-            protocol as c_int,
+            protocol.0,
+        ))
+    }
+}
+
+#[cfg(not(any(target_os = "redox", target_os = "wasi")))]
+pub(crate) fn socket_with(
+    domain: AddressFamily,
+    type_: SocketType,
+    flags: SocketFlags,
+    protocol: Protocol,
+) -> io::Result<OwnedFd> {
+    unsafe {
+        ret_owned_fd(libc::socket(
+            domain.0 as c_int,
+            type_.0 as c_int | flags.bits(),
+            protocol.0,
         ))
     }
 }
@@ -1524,7 +1540,7 @@ pub(crate) fn bind_v4(sockfd: BorrowedFd<'_>, addr: &SocketAddrV4) -> io::Result
     unsafe {
         ret(libc::bind(
             borrowed_fd(sockfd),
-            as_ptr(&addr.encode()).cast::<_>(),
+            as_ptr(&encode_sockaddr_v4(addr)).cast::<_>(),
             size_of::<libc::sockaddr_in>() as libc::socklen_t,
         ))
     }
@@ -1535,7 +1551,7 @@ pub(crate) fn bind_v6(sockfd: BorrowedFd<'_>, addr: &SocketAddrV6) -> io::Result
     unsafe {
         ret(libc::bind(
             borrowed_fd(sockfd),
-            as_ptr(&addr.encode()).cast::<_>(),
+            as_ptr(&encode_sockaddr_v6(addr)).cast::<_>(),
             size_of::<libc::sockaddr_in6>() as libc::socklen_t,
         ))
     }
@@ -1546,7 +1562,7 @@ pub(crate) fn bind_unix(sockfd: BorrowedFd<'_>, addr: &SocketAddrUnix) -> io::Re
     unsafe {
         ret(libc::bind(
             borrowed_fd(sockfd),
-            as_ptr(&addr.encode()).cast::<_>(),
+            as_ptr(&encode_sockaddr_unix(addr)).cast::<_>(),
             size_of::<libc::sockaddr_un>() as libc::socklen_t,
         ))
     }
@@ -1557,7 +1573,7 @@ pub(crate) fn connect_v4(sockfd: BorrowedFd<'_>, addr: &SocketAddrV4) -> io::Res
     unsafe {
         ret(libc::connect(
             borrowed_fd(sockfd),
-            as_ptr(&addr.encode()).cast::<_>(),
+            as_ptr(&encode_sockaddr_v4(addr)).cast::<_>(),
             size_of::<libc::sockaddr_in>() as libc::socklen_t,
         ))
     }
@@ -1568,7 +1584,7 @@ pub(crate) fn connect_v6(sockfd: BorrowedFd<'_>, addr: &SocketAddrV6) -> io::Res
     unsafe {
         ret(libc::connect(
             borrowed_fd(sockfd),
-            as_ptr(&addr.encode()).cast::<_>(),
+            as_ptr(&encode_sockaddr_v6(addr)).cast::<_>(),
             size_of::<libc::sockaddr_in6>() as libc::socklen_t,
         ))
     }
@@ -1579,7 +1595,7 @@ pub(crate) fn connect_unix(sockfd: BorrowedFd<'_>, addr: &SocketAddrUnix) -> io:
     unsafe {
         ret(libc::connect(
             borrowed_fd(sockfd),
-            as_ptr(&addr.encode()).cast::<_>(),
+            as_ptr(&encode_sockaddr_unix(addr)).cast::<_>(),
             size_of::<libc::sockaddr_un>() as libc::socklen_t,
         ))
     }
@@ -1626,7 +1642,10 @@ pub(crate) fn acceptfrom(sockfd: BorrowedFd<'_>) -> io::Result<(OwnedFd, SocketA
             storage.as_mut_ptr().cast::<_>(),
             &mut len,
         ))?;
-        Ok((owned_fd, decode_sockaddr(storage.as_ptr(), len)))
+        Ok((
+            owned_fd,
+            read_sockaddr_os(storage.as_ptr(), len.try_into().unwrap()),
+        ))
     }
 }
 
@@ -1649,7 +1668,10 @@ pub(crate) fn acceptfrom_with(
             &mut len,
             flags.bits(),
         ))?;
-        Ok((owned_fd, decode_sockaddr(storage.as_ptr(), len)))
+        Ok((
+            owned_fd,
+            read_sockaddr_os(storage.as_ptr(), len.try_into().unwrap()),
+        ))
     }
 }
 
@@ -1706,7 +1728,7 @@ pub(crate) fn getsockname(sockfd: BorrowedFd<'_>) -> io::Result<SocketAddr> {
             storage.as_mut_ptr().cast::<_>(),
             &mut len,
         ))?;
-        Ok(decode_sockaddr(storage.as_ptr(), len))
+        Ok(read_sockaddr_os(storage.as_ptr(), len.try_into().unwrap()))
     }
 }
 
@@ -1720,7 +1742,7 @@ pub(crate) fn getpeername(sockfd: BorrowedFd<'_>) -> io::Result<SocketAddr> {
             storage.as_mut_ptr().cast::<_>(),
             &mut len,
         ))?;
-        Ok(decode_sockaddr(storage.as_ptr(), len))
+        Ok(read_sockaddr_os(storage.as_ptr(), len.try_into().unwrap()))
     }
 }
 
@@ -1728,15 +1750,15 @@ pub(crate) fn getpeername(sockfd: BorrowedFd<'_>) -> io::Result<SocketAddr> {
 pub(crate) fn socketpair(
     domain: AddressFamily,
     type_: SocketType,
-    accept_flags: AcceptFlags,
+    flags: SocketFlags,
     protocol: Protocol,
 ) -> io::Result<(OwnedFd, OwnedFd)> {
     unsafe {
         let mut fds = MaybeUninit::<[OwnedFd; 2]>::uninit();
         ret(libc::socketpair(
             domain.0 as c_int,
-            type_.0 as c_int | accept_flags.bits(),
-            protocol as c_int,
+            type_.0 as c_int | flags.bits(),
+            protocol.0,
             fds.as_mut_ptr().cast::<c_int>(),
         ))?;
 
