@@ -112,6 +112,11 @@ pub(super) fn raw_fd<'a, Num: ArgNumber>(fd: c_int) -> ArgReg<'a, Num> {
     raw_arg(fd as c_uint as usize)
 }
 
+#[inline]
+pub(super) fn no_fd<'a, Num: ArgNumber>() -> ArgReg<'a, Num> {
+    raw_arg(-1_isize as usize)
+}
+
 #[inline]
 pub(super) fn slice_just_addr<'a, T: Sized, Num: ArgNumber>(v: &'a [T]) -> ArgReg<'a, Num> {
     raw_arg(v.as_ptr() as usize)
@@ -243,7 +248,7 @@ pub(super) fn oflags<'a, Num: ArgNumber>(oflags: OFlags) -> ArgReg<'a, Num> {
 }
 
 pub(super) fn oflags_for_open_how(oflags: OFlags) -> u64 {
-    oflags_bits(oflags) as u64
+    u64::from(oflags_bits(oflags))
 }
 
 #[inline]
