@@ -6,7 +6,7 @@
 //! file descriptor and close it ourselves.
 #![allow(unsafe_code)]
 
-use crate::io::{close, AsRawFd, FromRawFd};
+use crate::io::{close, AsRawFd, FromRawFd, IntoRawFd, RawFd};
 use io_lifetimes::{AsFd, BorrowedFd};
 #[cfg(not(io_lifetimes_use_std))]
 use io_lifetimes::{FromFd, IntoFd};
@@ -89,6 +89,31 @@ impl From<OwnedFd> for io_lifetimes::OwnedFd {
     }
 }
 
+impl AsRawFd for OwnedFd {
+    #[inline]
+    fn as_raw_fd(&self) -> RawFd {
+        self.inner.as_raw_fd()
+    }
+}
+
+impl IntoRawFd for OwnedFd {
+    #[inline]
+    fn into_raw_fd(self) -> RawFd {
+        let raw_fd = self.inner.as_fd().as_raw_fd();
+        forget(self);
+        raw_fd
+    }
+}
+
+impl FromRawFd for OwnedFd {
+    #[inline]
+    unsafe fn from_raw_fd(raw_fd: RawFd) -> Self {
+        Self {
+            inner: ManuallyDrop::new(io_lifetimes::OwnedFd::from_raw_fd(raw_fd)),
+        }
+    }
+}
+
 impl Drop for OwnedFd {
     #[inline]
     fn drop(&mut self) {
