@@ -11,8 +11,8 @@ fn test_mknodat() {
     let tmp = tempfile::tempdir().unwrap();
     let dir = openat(&cwd(), tmp.path(), OFlags::RDONLY, Mode::empty()).unwrap();
 
-    // Create a regular file. Not supported on FreeBSD.
-    #[cfg(not(target_os = "freebsd"))]
+    // Create a regular file. Not supported on FreeBSD or OpenBSD.
+    #[cfg(not(any(target_os = "freebsd", target_os = "openbsd")))]
     {
         mknodat(&dir, "foo", Mode::IFREG, 0).unwrap();
         let stat = statat(&dir, "foo", AtFlags::empty()).unwrap();
