@@ -9,3 +9,5 @@ mod priority;
 mod sched_yield;
 #[cfg(not(target_os = "wasi"))] // WASI doesn't have uname.
 mod uname;
+#[cfg(not(any(target_os = "redox", target_os = "wasi")))]
+mod working_directory;
